-- Usage: ./bounce -o bounce.gif -w 256

import Diagrams.Prelude
import Diagrams.Backend.Rasterific.CmdLine

main = mainWith animgif

animgif :: [(Diagram B R2, Int)]
animgif = zip diagrams delays

delay = 2

delays = repeat delay

frames = round (loopTime / (fromIntegral delay / 100))

diagrams = map (\f -> diagram (loopTime * fromIntegral f / fromIntegral frames)) [ 0 .. frames - 1 ]

diagram t
  = withEnvelope' (viewport t) (uncurry ellipseXY (ballAspect t) # fc grey # translate (r2(0, ballHeight t)) `atop` steps t) # bg white

withEnvelope' a b = withEnvelope (a `asTypeOf` b) b

viewport t = rect (2 * stepWidth) (bounceSize + stepHeight) # translate (r2(0, bounceSize / 2 - stepHeight * t / loopTime))

steps t = fromOffsets [ r2(stepWidth,0), r2(0,-stepHeight), r2(stepWidth,0), r2(0,-stepHeight), r2(stepWidth,0), r2(0,-stepHeight), r2(stepWidth,0), r2(0,-stepHeight), r2(-4*stepWidth,0) ] # closeLine # wrapTrail # pathFromTrail # stroke # fc grey # translate (r2(-1.5 * stepWidth - stepWidth * t / loopTime, stepHeight))


ballHeight t
  | t < flightTime = flightHeight t
  | otherwise = bounceHeight (t - flightTime) - stepHeight

ballAspect t
  | t < flightTime = (ballRadius, ballRadius)
  | otherwise = let h = bounceHeight (t - flightTime) / ballRadius in (ballRadius / sqrt h, ballRadius * h)

gravity = 9.8
stepWidth = 1
stepHeight = 0.5
ballRadius = stepHeight / 3
bounceSize = stepHeight * 3
bounceDepth = 0.25
mass = 1
energy0 = gravity * mass * (bounceSize - stepHeight)
energy1 = gravity * mass * bounceSize
velocity0 = sqrt (2 * energy0 / mass)
velocity1 = sqrt (2 * energy1 / mass)
height0 = ballRadius
height1 = ballRadius - stepHeight
-- flightAcceleration t = -gravity
-- flightVelocity t = velocity0 - gravity * t
flightHeight t = height0 + velocity0 * t - 0.5 * gravity * t * t
-- flightHeight t1 = height0 - stepHeight
-- 0.5 * gravity * t1^2 - velocity0 * t1 - stepHeight = 0
flightTime = (velocity0 + sqrt (velocity0^2 + 2 * gravity * stepHeight)) / (gravity)
bounceHeight t = ballRadius * (1 - bounceDepth * sin (beta * t) * exp (-gamma * t))
--bounceVelocity t = ballRadius * exp (-gamma * t) * (gamma * sin (beta t) - bounceDepth * beta * cos (beta t))
-- velocity1 = ballRadius * bounceDepth * beta
beta = velocity1 / (ballRadius * bounceDepth)
bounceTime = pi / beta
-- velocity1 = ballRadius * exp(- gamma * bounceTime) * bounceDepth * beta
gamma = log (velocity1 / (ballRadius * bounceDepth * beta)) / bounceTime
loopTime = flightTime + bounceTime
